﻿/*! Copyright Pulse-Eight Limited 2016 
	This file is licensed to you as part of your ownership of one of our systems.
	You may not reproduce this code in part or entirety without expressed permission of Pulse-Eight Limited.
	
	For licensing terms, please email licensing@pulse-eight.com
	https://www.pulse-eight.com/
*/

/* LANGUAGE en-GB */

var LANGUAGE_GENERIC_HEALTHY = 'Healthy';
var LANGUAGE_GENERIC_NO_CONNECTION = 'No Connection To Matrix';
var LANGUAGE_GENERIC_ERROR_FROM_SERVER = 'Error: Invalid response from server';
var LANGUAGE_SETTINGS_SAVED_SUCCESS = 'Settings Updated Successfully';
var LANGUAGE_SETTINGS_SAVED_FAILED = 'Error: Failed to Save Settings';
var LANGUAGE_HB_SENT = 'Monitoring Pulse Sent';
var LANGUAGE_HB_FORCE_FAILED = 'Error: Failed to send Monitoring Pulse';
var LANGUAGE_NETWORK_ERROR = 'Error: No response from server';
var LANGUAGE_GET_SETTINGS_FAILED = 'Error: Failed to fetch settings';
var LANGUAGE_ROUTING_FAILED = 'Error: Failed to change Input';
var LANGUAGE_ROUTING_SUCCESS = 'Input Changed';
var LANGUAGE_CEC_POWER_STATE_FAILED = 'Error: Failed to Set TV Power State';

var MODEL = {
	Bronze: 'FFMB44',
	Silver: 'FFMS44',
	Modular4: 'MM44',
	Modular8: 'MM88',
    Basic88: 'FFMB88'
}

var MATRIX_URL = '';
var ADMIN = false;
var PERIOD = 10000;
var MESSAGE_PERIOD = 5000;
var AJAX_TIMEOUT_MS = 5000;

var updateSystemDetailsIntervalActive = false;
var updateSystemDetailsInterval;

var doServerCall = function (url, success, error, complete) {
	$.ajax({
		url: MATRIX_URL + url,
		timeout: AJAX_TIMEOUT_MS,
		success: success || function () {},
		complete: complete || function () {},
		error: error || function (e, status) {
			switch (status) {
				case "timeout":
					displayBanner(false, LANGUAGE_GENERIC_NO_CONNECTION);
					break;
				default:
					displayBanner(false, LANGUAGE_GENERIC_ERROR_FROM_SERVER);
					break;
			}
		}
	});
}

var doServerPost = function(url, data, success, error, complete) {
	$.ajax({
		url: MATRIX_URL + url,
		timeout: AJAX_TIMEOUT_MS,
		type: 'POST',
		contentType: 'application/json',
		processData: false,
		data: JSON.stringify(data),
		success: success || function () { },
		complete: complete || function () { },
		error: error || function (e, status) {
			switch (status) {
				case "timeout":
					displayBanner(false, LANGUAGE_GENERIC_NO_CONNECTION);
					break;
				default:
					displayBanner(false, LANGUAGE_GENERIC_ERROR_FROM_SERVER);
					break;
			}
		}
	});
}

var getSystemDetails = function () {
	doServerCall('/System/Details',
		function (data) {
			if (data.Result) {
				localStorage.setItem('SYSTEM_VERSION', data.Version);
				localStorage.setItem('SYSTEM_MAC', data.MAC);
				localStorage.setItem('SYSTEM_SERIAL', data.Serial);
				localStorage.setItem('SYSTEM_MODEL', data.Model);
				localStorage.setItem('HARDWARE_VERSION', data.BoardRev);
				localStorage.setItem('LAST_STATUS', data.Status);
				localStorage.setItem('LAST_STATUSMESSAGE', data.StatusMessage);
				localStorage.setItem('VENDOR', data.VID);
				refreshStatus();
				$(document).trigger('settingsLoaded');
			}
		},
		function() {
			$('#SystemStatus').each(function(i, v) {
				$(v).val('Unavailable').removeClass('disabled-green disabled-yellow disabled-red').data('Status', 1);
				$(v).addClass('disabled-red');
			});
		}
	);
};

function refreshStatus() {
	$('#SystemStatus').each(function (i, v) {
		setStatus(v, localStorage.getItem('LAST_STATUS'), localStorage.getItem('LAST_STATUSMESSAGE'));
		if ($('#ClearStatusContainer') && localStorage.getItem('LAST_STATUS') != 0) {
			$('#ClearStatusContainer').show();
		}
	});
}

var setStatus = function (target, status, message) {
	$(target).html(message).removeClass('btn-warning btn-danger btn-success').attr('style', 'opacity: 0.65');
	switch (status) {
		case 0:
		case '0':
			$(target).addClass('btn-success');
			break;
		case 2:
		case '2':
			$(target).addClass('btn-warning');
			break;
		default:
			$(target).addClass('btn-danger');
			break;
	}
	$(target).data('Status', status).attr('style', 'opacity: 1');
	$(target).html(message || 'Unknown');
};

function validateIPaddress(inputText) {
	var ipformat = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
	return inputText.match(ipformat);
}

function displayBanner(success, message) {
	var alertCSS = success ? 'alert-success' : 'alert-danger';
	$('#AlertContainer').empty().append($('<div/>', { 'role': 'alert', 'class': 'alert col-md-8 col-md-offset-2 ' + alertCSS, 'style': 'z-index: 100' }).html(message).prepend($('<button/>', { 'type': 'button', 'class': 'close', 'data-dismiss': 'alert' })));
	setTimeout(clearBanner, MESSAGE_PERIOD);
}

function clearBanner() {
	$('#AlertContainer').empty();
}

function getTypeDescription(type) {
	switch (type) {
		case 'HDBT-TXLITE':
		case 'HDBT-RXLITE':
		case 'HDBT-TXLITE2':
		case 'HDBT-RXLITE2':
			return 'HDBaseT Lite';
		case 'HDBT-TXPRO':
		case 'HDBT-RXPRO':
		case 'HDBT-TXPRO2':
		case 'HDBT-RXPRO2':
			return 'HDBaseT';
		case 'NONE':
			return 'None/Missing';
		case 'HDMI':
			return 'HDMI';
	}
	return 'Unknown';
}

function getCSSForType(type) {
	switch (type) {
		case 'HDMI':
			return 'icon-hdmi';
		case 'HDMI2':
			return 'icon-hdmi2';
		case 'HDBT-TXPRO':
		case 'HDBT-RXPRO':
		case 'HDBT-TXPRO2':
		case 'HDBT-RXPRO2':
			return 'icon-hdbt-pro';
		case 'HDBT-TXLITE':
		case 'HDBT-RXLITE':
		case 'HDBT-TXLITE2':
		case 'HDBT-RXLITE2':
			return 'icon-hdbt-lite';
		default:
			return 'icon-missing';
	}
}

function getLinkQualityStatus(linkStatus) {
	switch (linkStatus) {
		case 0:
			return 'Excellent';
		case 1:
			return 'Good';
		case 2:
			return 'Poor';
		case 3:
			return 'Bad';
		case 4:
			return 'No Link / Unreliable connection';
	}
	return 'Unknown';
}

function isHDBT(type) {
	switch (type) {
		case 'HDBT-TXLITE':
		case 'HDBT-RXLITE':
		case 'HDBT-TXPRO':
		case 'HDBT-RXPRO':
		case 'HDBT-TXLITE2':
		case 'HDBT-RXLITE2':
		case 'HDBT-TXPRO2':
		case 'HDBT-RXPRO2':
			return true;
	}
	return false;
}

$(document).on('settingsLoaded', function () {
	if (localStorage.getItem('SYSTEM_MODEL')) {
		$('#SystemModel').text(localStorage.getItem('SYSTEM_MODEL') + ' - ' + localStorage.getItem('SYSTEM_VERSION'));
		$('#SystemSerial').text(localStorage.getItem('SYSTEM_SERIAL'));
		$('#SystemMAC').text(localStorage.getItem('SYSTEM_MAC'));
		$('#HardwareVersion').text(localStorage.getItem('HARDWARE_VERSION'));
	}
	switch (localStorage.getItem('VENDOR')) {
		case 'P8':
			$('#VendorLogo').html('<a href="https://www.pulse-eight.com/"><img src="/img/p8logo.png" alt="Pulse-Eight"/></a>');
			$('.vendorName').text('Pulse-Eight');
			break;
		case 'ANTI':
			$('#VendorLogo').html('<a style="margin-right: 10px;" href="http://www.antiference.co.uk/"><img src="/img/antilogo.png" alt="Antiference"/></a>');
			$('.vendorName').text('Antiference');
			break;
		case 'PHD':
			$('#VendorLogo').html('<a style="margin-right: 10px;" href="http://www.phdsolutions.co/"><img src="/img/phdlogo.png" alt="PhD Solutions"/></a>');
			$('.vendorName').text('PhD Solutions');
			break;
		case 'ZEK':
			$('#VendorLogo').html('<a style="margin-right: 10px;" href="http://www.zektor.com/"><img src="/img/zeklogo.png" alt="Zektor"/></a>');
			$('.vendorName').text('Zektor');
			break;
	}
});

$().ready(function (){
	$.ajaxSetup({
		cache: false
	});
	if (!updateSystemDetailsIntervalActive && updateSystemDetailsInterval == null) {
		updateSystemDetailsIntervalActive = true;
		updateSystemDetailsInterval = setInterval(getSystemDetails, PERIOD);
		getSystemDetails();
	}
});
