﻿var modulesUpdated = false;
var updateModules = function () {
	if (modulesUpdated) {
		return;
	}
	$('#firmware-updates').show();
	doServerCall('/Port/List',
		function(data) {
			modulesUpdated = true;
			//Remove Modules
			$('#TXNodes, #RXNodes').empty();
			//Populate Modules List
			$(data.Ports).each(function(i, v) {
				if (v.Mode == 'Output' && isHDBT(v.Type)) {
					doServerCall('/Port/Details/' + v.Mode + '/' + v.Bay,
						function(info) {
							displayNode(info, $('#TXNodes'), v.Bay);
							if (info.LinkStatus.toUpperCase() != 'NO LINK') {
								doServerCall('/Port/RemoteDetails/' + v.Mode + '/' + v.Bay,
									function(d) {
										displayNode(d, $('#RXNodes'), v.Bay);
									});
							}
						});
				}
			});
		});
};

var activeUpdate;
var activeBay;

function displayNode(v, moduleType, bay) {
	var moduleClass = v.Type != 'NONE' ? 'icon module-bay ' : '';
	switch (v.Mode) {
		case 'Output':
			moduleClass += 'module-output ';
			break;
	}
	moduleClass += getCSSForType(v.Type);
	if (v.Type == 'HDMI' || v.Type == 'HDMI2') {
		moduleClass += ' port-no-signal';
	} else {
		moduleClass += ' port-default';
	}
	if (moduleType) {
		var container = $('<div/>', {
			'id': v.Mode + '-' + bay,
			'data-bay': bay,
			'data-mode': v.Mode,
			'class': 'col-md-12 panel-white panel-body ' + moduleClass,
			'title': v.StatusDescription
		}).data('port', true);

		var videoDirectionLabel;
		switch (v.Mode) {
			case 'Output':
				videoDirectionLabel = 'Output (Transmitter) ';
				break;
			default:
				videoDirectionLabel = 'Output (Receiver) ';
				break;
		}
		if (v.FirmwareVersionAvailable && localStorage.getItem('SYSTEM_MODEL') != MODEL.Modular8) {
			var updateButton = $('<button/>', {
				'class': 'btn btn-wide btn-success',
				'style': 'float: right;'
			}).text('Update').on('click', function (e) {
				activeUpdate = v;
				activeBay = bay;
				$('#UpdateConfirm').modal('show');
				e.stopPropagation();
			});
			
			container.append(updateButton);
		}

		container.append($('<h4/>', {
			'style': 'color: #fff !important',
			'id': 'lbl-' + v.Mode + '-' + bay
		}).html(videoDirectionLabel + (bay + 1)));

		var versionText = 'Current Firmware Version: ' + v.FirmwareVersion;
		if (v.Type == 'UNKNOWN') {
			versionText = 'Error communicating with device';
		}
		container.append($('<p/>', { 'style': 'margin: 0' }).text(versionText));

		moduleType.append(container);
	}
}

$(document).on('settingsLoaded', function () {
	if (localStorage.getItem('SYSTEM_MODEL') != 'MM88') {
		updateModules();
	}
});

$('#StartUpdate').on('click', function() {
	if (activeUpdate != null) {
		doServerCall('/Port/UpdateValens/' + (activeUpdate.Mode || 'Output') + '/' + activeBay + '/' + (activeUpdate.HPDStatus != null),
			function() {
				$('#confirmContainer').hide();
				$('#progressContainer').show();
				setTimeout(updateProgress, 500);
			});
	}
});

$('#rebootMatrixBtn').on('click', function () {
	doServerCall('/System/Restart',function(data) {
		if (data.Result) {
			$('#RebootMatrixPopup').modal('show');
			$('#RebootProgressBar').attr('style', 'width: 0%');
			setTimeout(updateRebootProgress, 1000, 1);
		} else {
			alert('Something has gone wrong, please remove the power cable to reboot');
		}
	});
});

var updateRebootProgress = function (newPosition) {
	$('#RebootProgressBar').attr('style', 'width: ' + newPosition + '%');
	if (newPosition < 100) {
		newPosition += 1;
		setTimeout(updateRebootProgress, 1000, newPosition);
	} else {
		window.location.href = '/';
	}
}

var updateProgress = function () {
	doServerCall('/System/Details',
		function(data) {
			if (!data.Result && data.Percentage >= 0) {
				//Still Updating
				$('#UpdateStatus').text(data.state);
				$('#UpdateProgressBar').attr('style', 'width: ' + data.Percentage + '%');
				setTimeout(updateProgress, 500);
			} else {
				alert('Update has completed');
				location.reload(true);
			}
		});
}
