﻿/*! Copyright Pulse-Eight Limited 2016
	This file is licensed to you as part of your ownership of one of our systems.
	You may not reproduce this code in part or entirety without expressed permission of Pulse-Eight Limited.
	
	For licensing terms, please email licensing@pulse-eight.com
	https://www.pulse-eight.com/
*/

var healthTestInterval;
var healthTestRunning = false;

var getSystemHealth = function () {
	doServerCall('/System/Health',
		function(data) {
			$('#psu1Status, #psu2Status, #inputModulesStatus, #outputModulesStatus').removeClass('disabled-red disabled-green disabled-yellow');
			setStatus('#psu1Status', data.PSU1, data.PSU1Message);
			if (data.PSU2 != null) {
				$('#PSU2Container').show();
				setStatus('#psu2Status', data.PSU2, data.PSU2Message);
			} else {
				$('#PSU2Container').hide();
				$('#psu1StatusLabel').text('Power Supply');
			}
			var tempHealth = data.Temperature0 <= 50 ? 0 : data.Temperature0 <= 60 ? 2 : 1;
			setStatus('#SystemTemperature1', tempHealth, data.Temperature0 + '&deg; degrees');
			if (data.Temperature1 != null) {
				$('#Modular88Container').show();
				tempHealth = data.Temperature1 <= 50 ? 0 : data.Temperature1 <= 60 ? 2 : 1;
				setStatus('#SystemTemperature2', tempHealth, data.Temperature1 + '&deg; degrees');
			} else {
				$('#Modular88Container').hide();
			}
			setStatus('#inputModulesStatus', data.InputModules, data.InputModulesMessage);
			setStatus('#outputModulesStatus', data.OutputModules, data.OutputModulesMessage);
		});
};

function runHealthTest() {
	if (!healthTestRunning) {
		healthTestRunning = true;
		healthTestInterval = setInterval(getSystemHealth, PERIOD);
		getSystemHealth();
	}
}

var testRunning = false;
var IR_TEST_RUNTIME = 180;
var currentProgress = 0;
var portNoise = [];
var worstPortNoise = [];

function runIRNoiseTest() {
	testRunning = true;
	currentProgress = 0;
	worstPortNoise = [];
	updateIRTestProgress();
}

function formatSeconds(seconds) {
	var date = new Date(1970,0,1);
	date.setSeconds(seconds);
	return date.toTimeString().replace(/.*(\d{2}:\d{2}:\d{2}).*/, "$1");
}

function updateIRTestProgress() {
	if (testRunning) {
		currentProgress++;
		var percentageProgress = (currentProgress / IR_TEST_RUNTIME) * 100;
		updateIRTestProgressBars(percentageProgress);
		if (percentageProgress < 100) {
			doServerCall('/IR/History', function(data) {
				if (data.History && data.History.length) {
					portNoise = [];
					data.History.reverse();
					$.each(data.History, function (i, v) {
						if ((data.Uptime - v.Time) < 10) {
							portNoise[v.Bay + 1] = 1;
						} else if ((data.Uptime - v.Time) < 30) {
							portNoise[v.Bay + 1] = 2;
						} else {
							portNoise[v.Bay + 1] = 3;
						}
						if (worstPortNoise[v.Bay + 1] > portNoise[v.Bay + 1] || worstPortNoise[v.Bay + 1] == null) {
							worstPortNoise[v.Bay + 1] = portNoise[v.Bay + 1];
						}
					});
				}
				displayIRTestErrorBoxes();
				$('#IRNoiseTestTime').text(formatSeconds(IR_TEST_RUNTIME - currentProgress));
				setTimeout(updateIRTestProgress, 1000);
			});
		} else {
			$('#ReRunIRNoiseTestButton, #CloseIRNoiseTestButton').show();
			$('#CancelIRNoiseTestButton').hide();
		}
	}
}

function displayIRTestErrorBoxes() {
	for (var i = 1; i <= 8; i++) {
		if (worstPortNoise[i] != null) {
			var portFeedbackBox = $('#Output' + i + 'IRTestFeedback');
			switch(worstPortNoise[i]) {
				case 1:
					portFeedbackBox.text('The IR receiver connected to this port is picking up a lot of extra noise, this will severely disturb if not block entirely IR communication to the connected source device and/or control system, and/or mask control from other ports that are also watching the same source.');
					break;
				case 2:
					portFeedbackBox.text('The IR receiver connected to this port is picking up noise quite frequently, this will interupt on a regular basis IR communication to the connected source device and/or control system or prevent other ports from controlling the source device that this port is also watching.');
					break;
				case 3:
					portFeedbackBox.text('The IR receiver connected to this port is occasionally picking up stray IR messages, this may occasionally interupt IR communication to the connected source device and/or control system, if you repeat this test and it remains in this state, you may find it is tollerable, but we recommend relocating the IR emitter anyway.');
					break;
			}
			portFeedbackBox.show();
		}
	}
}

function updateIRTestProgressBars(percentage) {
	for (var i = 1; i <= 8; i++) {
		var progressBar = $('#Output' + i + 'IRTestProgressBar');
		progressBar.attr('style', 'width: ' + percentage + '%');
		switch(portNoise[i]) {
			case 1:
				progressBar.removeClass('progress-bar-info progress-bar-warning progress-bar-success').addClass('progress-bar-danger');
				break;
			case 2:
				progressBar.removeClass('progress-bar-info progress-bar-danger progress-bar-success').addClass('progress-bar-warning');
				break;
			case 3:
				progressBar.removeClass('progress-bar-danger progress-bar-warning progress-bar-success').addClass('progress-bar-info');
				break;
			default:
				progressBar.removeClass('progress-bar-info progress-bar-warning progress-bar-danger').addClass('progress-bar-success');
				break;
		}
		if (portNoise[i] == null) {
			progressBar.removeClass('progress-bar-info progress-bar-warning progress-bar-danger').addClass('progress-bar-success');
		}
	}
}

function abortIRNoiseTest() {
	testRunning = false;
}

$('#ClearStatusContainer').on('click', function () {
	$('#ClearStatusContainer').hide();
	doServerCall('/System/ClearSysStatus', function() { location.reload() });
});

$('#IRNoiseTestButton').on('click', function() {
	$('#IRNoiseTest').modal('show');
	if (localStorage.getItem('SYSTEM_MODEL') == 'MM88') {
		$('#Modular88IRTest').show();
	} else {
		$('#Modular88IRTest').hide();
	}
	runIRNoiseTest();
	return false;
});

var hdbtTestContinue = false;
var i = 0;
var testingPort = $('#HDBaseTTestOutputSelect').val();
var testMethod = 'mse';
var channelAHealth = [];
var channelBHealth = [];
var channelCHealth = [];
var channelDHealth = [];
var plot;

$('#HDBaseTTestOutputSelect').on('change', function () {
	if (testingPort != $('#HDBaseTTestOutputSelect').val()) {
		testingPort = $('#HDBaseTTestOutputSelect').val();
		resetHDBTTest();
	}
});

$('#HDBaseTTestMethodSelect').on('change', function () {
	testMethod = $('#HDBaseTTestMethodSelect').val();
	resetHDBTTest();
});

function resetHDBTTest() {
	i = 0;
	channelAHealth = [];
	channelBHealth = [];
	channelCHealth = [];
	channelDHealth = [];
}

function setupChart() {
	plot = $.plot($("#hdbtLinkQualityChart"), [
				{
					data: channelAHealth,
					label: "Channel A Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelBHealth,
					label: "Channel B Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelCHealth,
					label: "Channel C Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelDHealth,
					label: "Channel D Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}
	], {
		series: {
			lines: {
				show: true,
				lineWidth: 2,
				fill: false,
			},
			points: {
				show: true,
				radius: 2,
				lineWidth: 1
			},
			shadowSize: 2
		},
		grid: {
			hoverable: true,
			tickColor: "#eee",
			borderColor: "#eee",
			borderWidth: 1
		},
		colors: ["#d12610", "#37b7f3", "#52e136", "#ffcc00"],
		xaxis: {
			ticks: 11,
			tickDecimals: 0,
			tickColor: "#eee"
		},
		yaxis: {
			ticks: 11,
			tickDecimals: 0,
			tickColor: "#eee"
		}
	});
}

$('#HDBaseTLinkQualityButton').on('click', function () {
	if (localStorage.getItem('SYSTEM_MODEL') == 'MM88') {
		$('#HDBaseTTestOutputSelect').empty();
		for (let i = 0; i < 8; i++) {
			$('#HDBaseTTestOutputSelect').append($('<option/>', {
				val: i,
				text: 'Output ' + (i+1)
			}));
		}
	}
	$('#HDBaseTLinkQuality').modal('show');
	return false;
});

$('#HDBaseTLinkQuality').on('shown.bs.modal', function () {
	hdbtTestContinue = true;
	resetHDBTTest();
	setupChart();
	refreshData();
});

$('#HDBaseTLinkQuality').on('hidden.bs.modal', function() {
	hdbtTestContinue = false;
});

var refreshData = function () {
	doServerCall('/Port/Details/Output/' + testingPort,
		function(data) {
			if (data.Result) {
				if (data.LinkQuality) {
					$('#HDBaseTLinkTestError').hide();
					i++;
					var channelAData = testMethod == 'mse' ? data.LinkQuality.ChannelA : data.LinkQuality.ErrorsChannelA;
					var channelBData = testMethod == 'mse' ? data.LinkQuality.ChannelB : data.LinkQuality.ErrorsChannelB;
					var channelCData = testMethod == 'mse' ? data.LinkQuality.ChannelC : data.LinkQuality.ErrorsChannelC;
					var channelDData = testMethod == 'mse' ? data.LinkQuality.ChannelD : data.LinkQuality.ErrorsChannelD;

					channelAHealth.push([i, channelAData]);
					channelBHealth.push([i, channelBData]);
					channelCHealth.push([i, channelCData]);
					channelDHealth.push([i, channelDData]);
					if (i > 30) {
						channelAHealth.shift();
						channelBHealth.shift();
						channelCHealth.shift();
						channelDHealth.shift();
					}
				} else {
					$('#HDBaseTLinkTestError').show().text('Link Quality Unavailable');
				}
			}
		},
		null,
		function() {
			plot.setData([
				{
					data: channelAHealth,
					label: "Channel A Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelBHealth,
					label: "Channel B Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelCHealth,
					label: "Channel C Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}, {
					data: channelDHealth,
					label: "Channel D Health",
					lines: { lineWidth: 1 },
					shadowSize: 0
				}
			]);
			plot.setupGrid();
			plot.draw();
			if (hdbtTestContinue) {
				setTimeout(refreshData, 500);
			}
		});
}

$('#ReRunIRNoiseTestButton').on('click', function () {
	$('#ReRunIRNoiseTestButton, #CloseIRNoiseTestButton').hide();
	$('#CancelIRNoiseTestButton').show();
	runIRNoiseTest();
});

$('#CancelIRNoiseTestButton, #CloseIRNoiseTestButton').on('click', function () {
	$('#ReRunIRNoiseTestButton, #CloseIRNoiseTestButton').hide();
	$('#CancelIRNoiseTestButton').show();
	abortIRNoiseTest();
});

$().ready(function () {
	refreshStatus();
	runHealthTest();
});