﻿/*! Copyright Pulse-Eight Limited 2016
	This file is licensed to you as part of your ownership of one of our systems.
	You may not reproduce this code in part or entirety without expressed permission of Pulse-Eight Limited.
	
	For licensing terms, please email licensing@pulse-eight.com
	https://www.pulse-eight.com/
*/

document.networkTestRunning = false;
document.networkTestInterval = null;

function validateSubnetAddress(subnetAddress, ipAddress) {
	var subnetFormat = /^(?:(255|254|252|248|240|224|192|128|0+)(?:\.(?!$)|$)){4}/;
	return subnetAddress.match(subnetFormat);
}

function updateNetworkSettings() {
	doServerCall('/Network/Settings',function (data) {
		$('#ipAddress').val(data.IPAddress);
		$('#gatewayAddress').val(data.GatewayAddress);
		$('#subnetAddress').val(data.SubnetAddress);
		$('#primaryDNSAddress').val(data.PrimaryDNSAddress);
		$('#secondaryDNSAddress').val(data.SecondaryDNSAddress);
		$('#networkType').val(data.NetworkType).trigger('change');
		$('#networkSaveSettingsButton').attr('disabled', true);
	});
}

function runNetworkTest() {
	if (!document.networkTestRunning) {
		document.networkTestRunning = true;
		document.networkTestInterval = setInterval(networkTest, PERIOD);
		networkTest();
	}
}

var networkTest = function () {
	doServerCall('/Network/Test',
		function (data) {
			setStatus($('#NetworkStatus'), data.Result ? 0 : 1, data.Result ? LANGUAGE_GENERIC_HEALTHY : LANGUAGE_GENERIC_NO_CONNECTION);
		},
		function () {
			setStatus($('#NetworkStatus'), 1, LANGUAGE_NETWORK_ERROR);
		});
};

$(document).on('click', '#networkSaveSettingsButton', function() {
	$('#networkSaveSettingsButton').attr('disabled', true);
	var networkSettings;
	if ($('#networkType').val() == 0) {
		//Send Manual Settings
		networkSettings = {
			NetworkType: 0,
			IPAddress: $('#ipAddress').val(),
			SubnetAddress: $('#subnetAddress').val(),
			GatewayAddress: $('#gatewayAddress').val(),
			PrimaryDNSAddress: $('#primaryDNSAddress').val(),
			SecondaryDNSAddress: $('#secondaryDNSAddress').val()
		};
	} else {
		networkSettings = {
			NetworkType: 1
		};
	}
	doServerPost('/Network/SaveSettings', networkSettings,
		function(data) {
			if (data.Result) {
				displayBanner(true, LANGUAGE_SETTINGS_SAVED_SUCCESS);
				$('#networkSaveSettingsButton').attr('disabled', false);
			} else {
				displayBanner(false, data.ErrorMessage);
			}
		},
		function() {
			displayBanner(false, LANGUAGE_SETTINGS_SAVED_FAILED);
			$('#networkSaveSettingsButton').attr('disabled', false);
		});
});

$(document).on('change keyup focusout', '#ipAddress, #subnetAddress, #gatewayAddress, #primaryDNSAddress, #secondaryDNSAddress, #networkType', function(e) {
	var valid = true;
	if (!validateIPaddress($('#ipAddress').val())) {
		valid = false;
	}
	if (valid && !validateSubnetAddress($('#subnetAddress').val(), $('#ipAddress'))) {
		valid = false;
	}
	if (valid && !validateIPaddress($('#gatewayAddress').val())) {
		valid = false;
	}
	if (valid && !validateIPaddress($('#primaryDNSAddress').val())) {
		valid = false;
	}
	if (valid && !validateIPaddress($('#secondaryDNSAddress').val())) {
		valid = false;
	}

	/* IP Address & Gateway Address cannot be the same */
	if ($('#ipAddress').val() == $('#gatewayAddress').val()) {
		valid = false;
		$('#ipAddress, #gatewayAddress').addClass('disabled-red');
		if (e.type == 'change') {
			displayBanner(false, 'IP Address and Gateway Address cannot be identical');
		}
	} else {
		$('#ipAddress, #gatewayAddress').removeClass('disabled-red');
	}
	/* IP Address cannot be Primary DNS Address */
	if ($('#ipAddress').val() == $('#primaryDNSAddress').val()) {
		valid = false;
		$('#ipAddress, #primaryDNSAddress').addClass('disabled-red');
		if (e.type == 'change') {
			displayBanner(false, 'DNS Addresses cannot be the same as this units IP Address');
		}
	} else if ($('#primaryDNSAddress').hasClass('disabled-red')) {
		$('#ipAddress, #primaryDNSAddress').removeClass('disabled-red');
	}
	/* IP Address cannot be Secondary DNS Address */
	if ($('#ipAddress').val() == $('#secondaryDNSAddress').val()) {
		valid = false;
		$('#ipAddress, #secondaryDNSAddress').addClass('disabled-red');
		if (e.type == 'change') {
			displayBanner(false, 'DNS Addresses cannot be the same as this units IP Address');
		}
	} else if ($('#secondaryDNSAddress').hasClass('disabled-red')) {
		$('#ipAddress, #secondaryDNSAddress').removeClass('disabled-red');
	}
	if (valid) {
		$('#networkSaveSettingsButton').removeAttr('disabled');
	} else {
		$('#networkSaveSettingsButton').attr('disabled', true);
	}
});

$(document).on('change', '#networkType', function() {
	if ($('#networkType').val() == 0) {
		//Show Manual
		$('#ipAddress, #subnetAddress, #gatewayAddress, #primaryDNSAddress, #secondaryDNSAddress').removeAttr('readonly').fadeTo(400, 1);
	} else {
		//Hide Manual
		$('#ipAddress, #subnetAddress, #gatewayAddress, #primaryDNSAddress, #secondaryDNSAddress').attr('readonly', true).fadeTo(400, 0.3);
	}
});

$().ready(function() {
	refreshStatus();
	updateNetworkSettings();
	runNetworkTest();
});