﻿/*! Copyright Pulse-Eight Limited 2016
	This file is licensed to you as part of your ownership of one of our systems.
	You may not reproduce this code in part or entirety without expressed permission of Pulse-Eight Limited.
	
	For licensing terms, please email licensing@pulse-eight.com
	https://www.pulse-eight.com/
*/

var z = 10;
var FADE_VALUE = 0.3;
var updateTimeout;
var updateModulesInProgress = false;
var activeInputBay = -1;
var activeModuleId = -1;
var activeEditing = null;
var activeModuleMode = null;
var activeDisplayPowerStatus = null;
var activeModule = null;
var inputCount = 0;
var outputCount = 0;

var moduleInputName = [];
var moduleOutputName = [];

var updateModules = function() {
	//Fetch Modules
	if (!updateModulesInProgress) {
		updateModulesInProgress = true;
		doServerCall('/Port/List',
			function (data) {
				$('#firstTimeLoad').hide();
				$('#routingPanel').show();
				//Remove Modules
				$('#inputModules, #outputModules').empty();
				moduleInputName = [];
				moduleOutputName = [];
				inputCount = 0;
				outputCount = 0;
				//Populate Modules List
				$(data.Ports).each(function (i, v) {
					if (v.Hidden) {
						return;
					}
					var moduleType;
					var moduleClass = v.Type != 'NONE' ? 'icon module-bay ' : '';
					switch (v.Mode) {
						case 'Input':
							moduleType = $('#inputModules');
							moduleClass += 'module-input ';
							inputCount++;
							break;
						case 'Output':
							moduleType = $('#outputModules');
							moduleClass += 'module-output ';
							outputCount++;
							break;
					}
					moduleClass += getCSSForType(v.Type);
					switch (v.Status) {
						case 2:  // warning
							moduleClass += ' port-warning';
							break;
						case 3:  // no signal
							moduleClass += ' port-no-signal';
							v.StatusDescription = 'No HDMI Signal';
							break;
						case 4:  // module not present
							moduleClass += ' port-missing';
							v.StatusDescription = v.Mode + ' ' + (v.Name = 'Not Present');
							break;
						case 1:  // module fault
							moduleClass += ' port-fault';
							v.StatusDescription = v.Mode + ' is Faulty';
							break;
						default:
							moduleClass += ' port-default';
							break;
					}

					if (moduleType) {
						var container = $('<div/>', {
							'id': v.Mode + '-' + v.Bay,
							'data-bay': v.Bay,
							'data-mode': v.Mode,
							'style': 'position: relative',
							'class': 'clickable col-md-12 panel-white panel-body ' + moduleClass,
							'title': v.StatusDescription,
							'opacity': activeModule != null ? FADE_VALUE : 1
						}).data('port', true);

						//var videoDirectionLabel = '';
						var customName = v.Name || 'Manage';
						switch (v.Mode) {
							case 'Input':
								moduleInputName[v.Bay] = customName;
								//videoDirectionLabel = 'Source ';
								break;
							case 'Output':
								moduleOutputName[v.Bay] = customName;
								//videoDirectionLabel = 'Display ';
								break;
						}
						//videoDirectionLabel + (v.Bay + 1) + ' <i class="fa fa-angle-right"></i> '

						var virtualRemoteButton = $('<div/>', {
							'style': 'float: right; color: #fff; text-align: right; font-size: 1.5em; padding: 15px; width: 80px; position: absolute; top: 0px; right: 0px; height: 48px; ' + (v.Mode == 'Input' ? 'visibility: hidden;' : ''),
							'class': 'fa fa-th'
						}).on('click', function (e) {
							clearTimeout(updateTimeout);
							activeEditing = v;
							fetchActiveInputForOutput(activeEditing);
							$('#VirtualRemote').modal('show');
							e.stopPropagation();
						});

						container.append(virtualRemoteButton);

						container.append($('<h4/>', {
							'style': 'color: #fff !important',
							'id': 'lbl-' + v.Mode + '-' + v.Bay
						}).html(customName));

						container.on('click', function () {
							if ($(this).hasClass('noclick')) {
								$(this).removeClass('noclick');
							} else {
								getPortDetails(v);
							}
						});

						if (v.Mode == 'Input' && v.Type != 'Missing') {
							container.draggable({
								ghosting: true,
								opacity: 0.5,
								delay: 100,
								revert: true,
								start: function (e,ui) { $(this).css('z-index', z++).addClass('noclick') }
							});
						} else if (v.Type != 'Missing') {
							container.droppable({
								drop: function (e, ui) {
									doServerCall('/Port/Set/' + $(ui.draggable).data('bay') + '/' + $(this).data('bay'),
										function (d) {
											if (d.Result) {
												displayBanner(true, 'Input Changed');
											} else {
												displayBanner(false, 'Error: Routing Failed');
											}
											if (activeModule != null) {
												setTimeout(function() { getPortDetails(activeModule, true); }, 1000);
											}
										});
								}
							});
						}
						moduleType.append(container);
					}
				});
				if (activeModule != null) {
					getPortDetails(activeModule, true);
				}
			},
			null, //error
			function() {
				updateModulesInProgress = false;
				clearTimeout(updateTimeout);
				updateTimeout = setTimeout(updateModules, PERIOD);
			});
	}
};

function fetchActiveInputForOutput(selectedPort) {
	activeDisplayPowerStatus = null;
	$('#VMPower').css('color', '#8e8e93');
	doServerCall('/Port/Details/' + selectedPort.Mode + '/' + selectedPort.Bay,
		function (data) {
			$('#VMPower').addClass('fa-power-off').removeClass('fa-circle-o-notch fa-spin');
			activeInputBay = data.ReceiveFrom;
			switch (data.DPS) {
				case 0:
				case '0':
				case 2:
				case '2':
					activeDisplayPowerStatus = 1; //On
					$('#VMPower').css('color', '#179caf');
					break;
				case 1:
				case '1':
				case 3:
				case '3':
					activeDisplayPowerStatus = 0; //Off
					$('#VMPower').css('color', '#C82E29');
					break;
				default:
					activeDisplayPowerStatus = 2;
					$('#VMPower').css('color', 'orange');
					break;
			}
			if (activeDisplayPowerStatus != null) {
				$('#VMPowerBtn').show();
			} else {
				$('#VMPowerBtn').hide();
			}

			populateVMSinkList();
			populateVMWatchList();
		});
}

function populateVMSinkList() {
	//TEMP HACK
	//$('#VMSinkDeviceList').empty();
	//$('#VMSinkDeviceList').append($('<option/>').text(activeEditing.Name));
	$('#VMSink').text(activeEditing.Name);
}

function populateVMWatchList() {
	$('#VMSourceDeviceList').empty();
	$.each(moduleInputName, function (i, v) {
		if (v !== undefined) {
			var option = $('<option/>').attr('value', i).text(v);
			if (activeInputBay == i) {
				option.attr('selected', 'selected');
			}
			$('#VMSourceDeviceList').append(option);
		}
	});
}

function getPortDetails(selectedPort, refresh) {
	if (activeModule == null || activeModule.Bay != selectedPort.Bay || activeModule.Mode != selectedPort.Mode || refresh) {
		doServerCall('/Port/Details/' + selectedPort.Mode + '/' + selectedPort.Bay,
			function (data) {
				//show
				$('div:data(port)').each(function (t, u) {
					u = $(u);
					var fadeValue = ((u.data('bay') != selectedPort.Bay || u.data('mode') != selectedPort.Mode) ? FADE_VALUE : 1);

					switch (selectedPort.Mode) {
						case 'Input':
							if (u.data('mode') != 'Input' && $.inArray(u.data('bay'), data.TransmissionNodes) > -1) {
								fadeValue = 1;
							}
							break;
						case 'Output':
							if (u.data('mode') == 'Input' && u.data('bay') == data.ReceiveFrom) {
								fadeValue = 1;
							}
							break;
					}

					$(u).fadeTo(refresh ? 0 : 500, fadeValue);
				});

				activeModule = selectedPort;
			});
	} else {
		//hide
		activeModule = null;
		$('div:data(port)').each(function (t, u) {
			$(u).fadeTo(500, 1);
		});
	}
}

$(document).ready(function() {
	refreshStatus();
	updateModules();
});