﻿/*! Copyright Pulse-Eight Limited 2016
	This file is licensed to you as part of your ownership of one of our systems.
	You may not reproduce this code in part or entirety without expressed permission of Pulse-Eight Limited.
	
	For licensing terms, please email licensing@pulse-eight.com
	https://www.pulse-eight.com/
*/

var z = 10;
var FADE_VALUE = 0.3;
var updateTimeout;
var updateModulesInProgress = false;

var activeModuleId = -1;
var activeEditing = null;
var activeModuleMode = null;
var activeModule = null;
var inputCount = 0;
var outputCount = 0;
var mirrorCount = 0;

var moduleInputName = [];
var moduleInputHidden = [];
var moduleOutputName = [];
var moduleOutputHidden = [];

var updateModules = function() {
	//Fetch Modules
	if (!updateModulesInProgress) {
		updateModulesInProgress = true;
		doServerCall('/Port/List',
			function(data) {
				//Remove Modules
				$('#inputModules, #outputModules, #mirrorModules').empty();
				moduleInputName = [];
				moduleOutputName = [];
				inputCount = 0;
				outputCount = 0;
				mirrorCount = 0;
				//Populate Modules List
				$(data.Ports).each(function(i, v) {
					var moduleType;
					var moduleClass = v.Type != 'NONE' ? 'icon module-bay ' : '';
					switch (v.Mode) {
					case 'Input':
						moduleType = $('#inputModules');
						moduleClass += 'module-input ';
						inputCount++;
						break;
					case 'Output':
						moduleType = $('#outputModules');
						moduleClass += 'module-output ';
						outputCount++;
						break;
					case 'Mirror':
						moduleType = $('#mirrorModules');
						moduleClass += 'module-output ';
						mirrorCount++;
						break;
					}
					moduleClass += getCSSForType(v.Type);
					switch (v.Status) {
					case 2: // warning
						moduleClass += ' port-warning';
						break;
					case 3: // no signal
						moduleClass += ' port-no-signal';
						v.StatusDescription = 'No HDMI Signal';
						break;
					case 4: // module not present
						moduleClass += ' port-missing';
						v.StatusDescription = v.Mode + ' ' + (v.Name = 'Not Present');
						break;
					case 1: // module fault
						moduleClass += ' port-fault';
						v.StatusDescription = v.Mode + ' is Faulty';
						break;
					default:
						moduleClass += ' port-default';
						break;
					}

					if (moduleType) {
						var container = $('<div/>', {
							'id': v.Mode + '-' + v.Bay,
							'data-bay': v.Bay,
							'data-mode': v.Mode,
							'class': 'clickable col-md-12 panel-white panel-body ' + moduleClass,
							'title': v.StatusDescription,
							'opacity': activeModule != null ? FADE_VALUE : 1
						}).data('port', true);

						var videoDirectionLabel;
						var customName = v.Name || 'Manage';
						switch (v.Mode) {
						case 'Input':
							moduleInputName[v.Bay] = customName;
							moduleInputHidden[v.Bay] = v.Hidden;
							videoDirectionLabel = 'Source ';
							break;
						case 'Output':
							moduleOutputName[v.Bay] = customName;
							moduleOutputHidden[v.Bay] = v.Hidden;
							videoDirectionLabel = 'Display ';
							break;
						case 'Mirror':
							customName = customName + ' (Mirroring Output ' + (v.Bay + 1) + ')';
							videoDirectionLabel = 'Mirror Display ';
							break;
						}

						if (v.Hidden) {
							customName += ' [Hidden]';
						}

						var propertiesButton = $('<div/>', {
							'style': 'float: right; color: #fff; font-size: 1.5em',
							'class': 'fa fa-' + (v.Mode == 'Input' ? 'server' : 'desktop')
						}).on('click', function(e) {
							clearTimeout(updateTimeout);
							activeEditing = v;
							$('#PortTypeTitle').text(videoDirectionLabel + (v.Bay + 1));
							$('#PortName').val(v.Name).attr('placeholder', v.Name);
							populatePortDetailsModal();
							$('#EditDetails').modal('show');
							e.stopPropagation();
						});
						container.append(propertiesButton);

						container.append($('<h4/>', {
							'style': 'color: #fff !important',
							'id': 'lbl-' + v.Mode + '-' + v.Bay
						}).html(videoDirectionLabel + (v.Bay + 1) + ' <i class="fa fa-angle-right"></i> ' + customName));

						container.on('click', function() {
							if ($(this).hasClass('noclick')) {
								$(this).removeClass('noclick');
							} else {
								getPortDetails(v);
							}
						});

						if (v.Mode == 'Input' && v.Type != 'Missing') {
							container.draggable({
								ghosting: true,
								opacity: 0.5,
								delay: 100,
								revert: true,
								start: function(e, ui) { $(this).css('z-index', z++).addClass('noclick') }
							});
						} else if (v.Type != 'Missing') {
							container.droppable({
								drop: function (e, ui) {
									doServerCall('/Port/Set/' + $(ui.draggable).data('bay') + '/' + $(this).data('bay'),
										function(d) {
											if (d.Result) {
												displayBanner(true, 'Input Changed');
											} else {
												displayBanner(false, 'Error: Routing Failed');
											}
											if (activeModule != null) {
												setTimeout(function() { getPortDetails(activeModule, true); }, 1000);
											}
										});
								}
							});
						}
						moduleType.append(container);
					}
				});
				if (activeModule != null) {
					getPortDetails(activeModule, true);
				}
			}, null,
			function() {
				clearTimeout(updateTimeout);
				updateTimeout = setTimeout(updateModules, PERIOD);
				updateModulesInProgress = false;
			});
	}
};

$('#ViewDetails, #EditDetails').on('hidden.bs.modal', function () {
	activeModule = null;
	updateTimeout = setTimeout(updateModules, PERIOD);
});


function getPortDetails(selectedPort, refresh) {
	if (activeModule == null || activeModule.Bay != selectedPort.Bay || activeModule.Mode != selectedPort.Mode || refresh) {
		doServerCall('/Port/Details/' + selectedPort.Mode + '/' + selectedPort.Bay,
			function (data) {
				//show
				$('div:data(port)').each(function (t, u) {
					u = $(u);
					var fadeValue = ((u.data('bay') != selectedPort.Bay || u.data('mode') != selectedPort.Mode) ? FADE_VALUE : 1);

					switch (selectedPort.Mode) {
						case 'Input':
							if (u.data('mode') != 'Input' && $.inArray(u.data('bay'), data.TransmissionNodes) > -1) {
								fadeValue = 1;
							}
							break;
						case 'Output':
						case 'Mirror':
							if (u.data('mode') == 'Input' && u.data('bay') == data.ReceiveFrom) {
								fadeValue = 1;
							}
							if (selectedPort.Mode == 'Mirror' && u.data('mode') == 'Output' && u.data('bay') == data.Bay) {
								fadeValue = 1;
							}
							break;
					}

					$(u).fadeTo(refresh ? 0 : 500, fadeValue);
				});

				activeModule = selectedPort;
			});
	} else {
		//hide
		activeModule = null;
		$('div:data(port)').each(function (t, u) {
			$(u).fadeTo(500, 1);
		});
	}
}

$(document).on('pduStateUpdated', function() {
	if (canManagePower() && activeEditing.Mode == 'Input') {
		$('#rebootSourceBtn').show();
	} else {
		$('#rebootSourceBtn').hide();
	}
});

$('#rebootSourceBtn').on('click', function () {
	if (activeEditing.Mode == 'Input') {
		$('#rebootSourceBtn').text('Rebooting...');
		setPortState(activeEditing.Bay, 'reboot', false);
		setTimeout(function() {
			$('#rebootSourceBtn').text('Restart Device');
		}, 10000);
	}
});

function populatePortDetailsModal() {
	$('#EDIDProfileContainer, #RoutingControls, #PortHiddenContainer, #HDBTContainer, #RemoteDetailsButton, #LinkQualityContainer, #CableLengthContainer, #CECControls, #AudioModeContainer').hide();
	doServerCall('/Port/Details/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function (data) {
			if (canManagePower() && activeEditing.Mode == 'Input') {
				$('#rebootSourceBtn').show();
				$('#rebootSourceBtn').text('Restart Device');
			} else {
				$('#rebootSourceBtn').hide();
			}
			
			var powerStatus = -1;
			//DPS = DevicePowerStatus
			switch (data.DPS) {
				case 0:
				case '0':
				case 2:
				case '2':
					powerStatus = 0;
					break;
				case 1:
				case '1':
				case 3:
				case '3':
					powerStatus = 1;
					break;
			}

			if (typeof data.EDIDRead != 'undefined' && !data.EDIDRead) {
				$('#EDIDFailureContainer').show();
			} else {
				$('#EDIDFailureContainer').hide();
			}

			if (powerStatus != -1) {
				$('#CECControls').empty().show();
				var cecContainer = $('<div/>', { 'class': 'col-sm-9' });
				var cecOptions = $('<select/>', { 'style': 'width: 50px'});

				var onOption = $('<option/>', { 'value': 0 }).html('On');
				var offOption = $('<option/>', {'value': 1 }).html('Off');

				(powerStatus ? offOption : onOption).attr('selected', 'selected');

				cecOptions.append(onOption);
				cecOptions.append(offOption);
				cecOptions.change(function () {
					var direction = $(cecOptions).val() == 1 ? 'Off' : 'On';
					doServerCall('/CEC/' + direction + '/' + activeEditing.Mode + '/' + activeEditing.Bay,
						function (cecResult) {
							if (cecResult.Result) {
								displayBanner(true, 'Device Powering ' + direction);
							} else {
								displayBanner(false, 'Warning: Device did not accept Power State change request');
							}
						});
				});
				cecContainer.append(cecOptions);
				$('#CECControls').prepend(cecContainer);
				$('#CECControls').prepend($('<label/>', { 'class': 'col-sm-3 control-label' }).html('Power Status'));
			}

			if (data.Mode == 'Input' || data.Mode == 'Output') {
				var hiddenToggle = $('<input/>', {
					'type': 'checkbox',
					'class': 'js-switch',
					'id': 'PortHidden'
				});
				var hidden = (data.Mode == 'Input' ? moduleInputHidden : moduleOutputHidden)[data.Bay];

				$(hiddenToggle).prop('checked', hidden ? 'checked' : '');
				$('#PortHiddenBox').html('').append(hiddenToggle);
				$('#PortHiddenContainer').show();
			}

            if (data.Mode == 'Output') {
                $('#EDIDDetailsOutputButton').show();
            } else {
                $('#EDIDDetailsOutputButton').hide();
            }

			if (data.HDCP) {
				$('#HDCPVersionContainer').show();
				$('#HDCPVersion').text(getHDCPVersion(data.HDCP));
			} else {
				$('#HDCPVersionContainer').hide();
			}

			if (isHDBT(data.Type)) {
				$('#HDBTContainer').show();
				$('#FirmwareVersion').html(data.FirmwareVersion);
				$('#LinkStatus').removeClass('has-warning');
				switch (data.LinkStatus) {
					case 'NO LINK':
						$('#LinkStatus').html('No Link').addClass('has-warning');
						break;
					case 'ETH FALLBACK':
						$('#LinkStatus').html('10/100 Ethernet Mode');
						break;
					case 'HDBASE-T':
					case 'HD-BASET':
					case 'HDBaseT':
						$('#CableLengthContainer').show();
						$('#CableLength').html(data.EstimatedCableLength ? data.EstimatedCableLength >= 110 ? '> 110m' : data.EstimatedCableLength + 'm~' : '< 20m');
						if (data.LinkQuality != null) {
							$('#LinkQualityContainer').show();
							$('#LinkQuality').attr('title', 'Channel A: ' + data.LinkQuality.ChannelA + 'dB, Channel B: ' + data.LinkQuality.ChannelB + 'dB, Channel C: ' + data.LinkQuality.ChannelC + 'dB, Channel D: ' + data.LinkQuality.ChannelD + 'dB').html(getLinkQualityStatus(data.LinkQuality.Status));

							if (data.LinkQuality.ChannelA > -13 || data.LinkQuality.ChannelB > -13 || data.LinkQuality.ChannelC > -13 || data.LinkQuality.ChannelD > -13 || data.LinkQuality.Status > 2) {
								$('#LinkQuality').addClass('has-error');
							}
						}
						$('#LinkStatus').html('HDBaseT (Active)');
						$('#RemoteDetailsButton').show();
						break;
					default:
						$('#LinkStatus').html(data.LinkStatus);
						break;
				}
			}
			
			switch (data.Mode) {
				case 'Input':
					$('#EDIDProfileContainer').show();
					var foundSetting = false;
					var defaultProfileSetting = 11;
					var edidProfileSelect = $('#EDIDProfile').html('');
					var defaultOption = $('<option/>', {
						'value': 11
					}).html('Default (Lowest Common Denominator) [Recommended]');
					var profiles = {0 :  {tpl:11, name:'Default (Lowest Common Denominator) [Recommended]'},
							100: {tpl:0,  name:'1080p EDID Profile'},
							101: {tpl:2,  name:'4K EDID Profile'},
							102: {tpl:3,  name:'1080p/5.1 EDID Profile'},
							103: {tpl:5,  name:'1080p/7.1 EDID Profile'},
							104: {tpl:4,  name:'720p EDID Profile'}};
					for (var e = 0; e < outputCount; e++) {
						var p = 101 + e;
						var entry = 10 + e;
						profiles[entry] = {tpl:p, name:'Clone Display ' + (e + 1) + ' EDID'};
					}

					for (var profile in profiles) {
						var edidOption = $('<option/>', {
							'value': profiles[profile].tpl
						}).html(profiles[profile].name);
						if (data.EdidProfile == profiles[profile].tpl) {
							edidOption.attr('selected', 'selected');
							foundSetting = true;
						}
						edidProfileSelect.append(edidOption);
					}
					if (!foundSetting) {
						var customOption = $('<option/>', {
							'value': data.EdidProfile,
							'selected': 'selected'
						}).html('Custom Profile');
						edidProfileSelect.append(customOption);
					}
					break;
				case 'Output':
					var routingSourceContainer = $('<div/>', { 'class': 'col-sm-9' });

					var sourceOptions = $('<select/>', {'class':'form-control'});
					for (var i = 0; i < inputCount; i++) {
						var option = $('<option/>', {
							'value': i
						}).html(moduleInputName[i]);
						if ((data.TransmissionNodes != null && data.TransmissionNodes[0] == i) || data.ReceiveFrom == i) {
							option.attr('selected', 'selected');
						}
						sourceOptions.append(option);
					}

					sourceOptions.change(function () {
						doServerCall('/Port/Set/' + $(sourceOptions).val() + '/' + activeEditing.Bay,
							function(d) {
								if (d.Result) {
									displayBanner(true, LANGUAGE_ROUTING_SUCCESS);
								}
							});
					});
					routingSourceContainer.append(sourceOptions);
					$('#RoutingControls').empty().show();
					$('#RoutingControls').prepend(routingSourceContainer);
					$('#RoutingControls').prepend($('<label/>', { 'class': 'col-sm-3 control-label' }).html('Video From'));
					break;
				case 'Mirror':
					$('#modulePropertiesText').text('Output Mirror Port ' + (activeModuleId + 1) + ' Properties');
					$('#moduleTransmissionContainer').hide();
					$('#AudioModeContainer').show();

					$('#AudioOnlyContainer').html('');
					var audioToggle = $('<input/>', {
						'type': 'checkbox',
						'class': 'js-switch',
						'id': 'AudioOnly'
					}).change(function () {
						doServerPost('/Port/SetAudio/' + activeEditing.Mode + '/' + activeEditing.Bay, { AudioOnly: $('#AudioOnly').is(':checked') },
							function (data) {
								if (data.Result) {
									displayBanner(true, LANGUAGE_SETTINGS_SAVED_SUCCESS);
								}
							});
					});
					audioToggle.prop('checked', data.AudioOnly ? 'checked' : '');
					$('#AudioOnlyContainer').append(audioToggle);
					$('#moduleProperties').slideDown();
					break;
				default:
					$('#modulePropertiesText').text('Module ' + (activeModuleId + 1) + ' Properties');
					$('#moduleTransmissionContainer').hide();
					$('#moduleProperties').slideDown();
					break;
			}
			$.each($('.js-switch'), function (i, v) {
				var sw = new Switchery(v);
			});
		});
}

function getHDCPVersion(value) {
	switch (value) {
		case 0:
			return "Not Supported";
		case 1:
			return "1.x";
		case 2:
			return "2.0";
		case 3:
			return "2.1";
		case 4:
			return "2.2";
	}
	return "Unknown";
}

$(document).on('click', '#EDIDDetailsButton, #btnEDIDDetailsOutput', function () {
	$('#EditDetails').modal('hide');
	$('#MainTitle').html($('#PortTypeTitle').html() + ' EDID Details');
	$('#MainDescription').html('Advanced EDID Information');
	$('#PortContainer').hide();
	$('#EDIDDetailsContainer, #EDIDCloseButton').show();
	doServerCall('/EDID/Details/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function(data) {
			if (data.Result) {
				$('#EDIDVersion').html(data.Version);
				$('#EDIDMonitorName').html(data.MonitorName);
				$('#EDIDVendor').html(data.Vendor);
				$('#EDIDModel').html(data.Model);
				$('#EDIDSize').html(data.ImageSize);
				$('#EDIDColourFormats').html(data.ColourFormats);
			}
		});
	doServerCall('/EDID/Timings/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function (data) {
			var timings1 = $('<ul/>');
			var timings2 = $('<ul/>');
			$(data.SupportedTimings).each(function (i, v) {
				(i % 2 ? timings1 : timings2).append($('<li/>').html(v));
			});
			$('#EDIDTimings1').html('').append(timings1);
			$('#EDIDTimings2').html('').append(timings2);
		});
	doServerCall('/EDID/HDMI/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function (data) {
			if (data.Result) {
				if (data.MaxTMDS) {
					$('#EDIDTMDS').html(data.MaxTMDS + 'Mhz');
				} else {
					$('#EDIDTMDS').html('Not Specified');
				}
				$('#EDIDHDMIOptions').html(data.Options);
			} else {
				$('#EDIDTMDS').html('Currently Unavailable');
			}
		});
	doServerCall('/EDID/Audio/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function (data) {
			if (data.Result) {
				$('#EDIDAudioSpeakers').html(data.Speakers);
				var audioCodecs = $('<ul/>');
				$(data.Codecs).each(function(i, v) {
					var codec = $('<li/>').html('Format: ' + v.Format + '<br />Channels: ' + v.Channels + '<br />Frequencies: ' + v.Frequencies + '<br />' + v.Details);
					audioCodecs.append(codec);
				});
				$('#EDIDAudioCodecs').html('').append(audioCodecs);
			} else {
				$('#EDIDAudioSpeakers').html('Currently Unavailable');
			}
		});
});

$(document).on('click', '#RemoteDetailsButton', function () {
	$('#EditDetails').modal('hide');
	doServerCall('/Port/RemoteDetails/' + activeEditing.Mode + '/' + activeEditing.Bay,
		function (data) {
			$('#RemotePortTypeTitle').html($('#PortTypeTitle').html() + ' Remote End Details');
			$('#RemoteType').html(getTypeDescription(data.Type));
			$('#RemoteFirmwareVersion').html(data.FirmwareVersion);
			$('#RemoteOperationMode').html(data.OpMode);
			var linkStatus = data.LinkStatus;
			switch (linkStatus) {
				case 'HD-BASET':
					linkStatus = 'HDBaseT';
					break;
			}
			$('#RemoteLinkType').html(linkStatus);
			$('#RemoteHPD').html(data.HPDStatus ? 'Enabled' : 'Disabled');
			var hdmiStatus = data.HDMIStatus;
			switch (data.HDMIStatus) {
				case 'None':
					hdmiStatus = 'No Signal';
					break;
			}
			$('#RemoteHDMI').html(hdmiStatus);
			$('#EditRemoteDetails').modal('show');
		});
});

$('#EditRemoteDetails, #EDIDDetails').on('hidden.bs.modal', function () {
	$('#EditDetails').modal('show');
});

$('#EDIDCloseButton').on('click', function () {
	$('#EditDetails').modal('show');
	$('#EDIDCloseButton').hide();
	$('#MainTitle').text('Video Routing');
	$('#MainDescription').text('select a port to see its routing path');
	$('#EDIDDetailsContainer').hide();
	$('#PortContainer').show();
});

$(document).on('click', '#ApplyPortEditButton', function() {
	if (activeEditing == null) {
		return;
	}

	if ($('#PortName').val() != $('#PortName').attr('placeholder')) {
		doServerPost('/Port/SetName/' + activeEditing.Mode + '/' + activeEditing.Bay + '/',
			{ Name: $('#PortName').val() },
			function (data) {
			    if (data.Result) {
			        try {
			            doRefresh(true);
			        } catch (err) {}
			        displayBanner(true, LANGUAGE_SETTINGS_SAVED_SUCCESS);
				} else {
					displayBanner(false, LANGUAGE_SETTINGS_SAVED_FAILED);
				}
			});
	}
	if (activeEditing.Mode == 'Input') {
		doServerCall('/Port/EDID/Set/' + $('#EDIDProfile').val() + '/' + activeEditing.Bay,
			function(data) {
				if (data.Result) {
					displayBanner(true, LANGUAGE_SETTINGS_SAVED_SUCCESS);
				} else {
					displayBanner(false, LANGUAGE_SETTINGS_SAVED_FAILED);
				}
			});
	}
	doServerCall('/Port/sethide/' + ($('#PortHidden').is(':checked') ? 1 : 0) + '/' + activeEditing.Mode + '/' + activeEditing.Bay);
	
	setTimeout(updateModules, 1000);
	$('#EditDetails').modal('hide');
});

$('#RemixEDIDButton').on('click', function () {
	$('#EDIDFailurePrompt').hide();
	doServerCall('/Port/EDID/Remix', function(data) {
		if (data.Result) {
			$('#EDIDRemixStarted').show();
			setTimeout(function() { location.reload(true); }, 60000);
		}
	});
});

$(document).ready(function() {
	refreshStatus();
	updateModules();
});
